unit Frustum;

interface

uses
    Windows,
    GL,
    GLU,
    BaseTypes,
    BaseMath;

const
     plane_A=0;
     plane_B=1;
     plane_C=2;
     plane_D=3;

     frustum_RIGHT=0;
     frustum_LEFT=1;
     frustum_BOTTOM=2;
     frustum_TOP=3;
     frustum_BACK=4;
     frustum_FRONT=5;

type
    TFrustumValues=array [0..5,0..3] of TFloat;

    TFrustum=class
                  private
                  proj,
                  modl,
                  clip:array [0..15] of TFloat;
                  fFrustum:TFrustumValues;
                  protected
                  public
                  procedure CalculateFrustum;
                  function PointInFrustum(aX,aY,aZ:TFloat):boolean;
                  function SphereInFrustum(aX,aY,aZ,aRadius:TFloat):boolean;
                  function CubeInFrustum(aX,aY,aZ,aSize:TFloat):boolean;
    end;

procedure NormalizePlane(var frustum:TFrustumValues;side:integer);

implementation

procedure NormalizePlane(var frustum:TFrustumValues;side:integer);
var
   magnitude:TFloat;
begin
     magnitude:=sqrt(sqr(frustum[side][plane_A])+
                     sqr(frustum[side][plane_B])+
                     sqr(frustum[side][plane_C]));

     frustum[side][plane_A]:=frustum[side][plane_A]/magnitude;
     frustum[side][plane_B]:=frustum[side][plane_B]/magnitude;
     frustum[side][plane_C]:=frustum[side][plane_C]/magnitude;
     frustum[side][plane_D]:=frustum[side][plane_D]/magnitude;
end;

{ TFrustum }

procedure TFrustum.CalculateFrustum;
var
   temp:TMatrix;
begin
     glGetFloatv(GL_PROJECTION_MATRIX,@proj[0]);

     glGetFloatv(GL_MODELVIEW_MATRIX,@modl[0]);

     clip[ 0] := modl[ 0] * proj[ 0] + modl[ 1] * proj[ 4] + modl[ 2] * proj[ 8] + modl[ 3] * proj[12];
     clip[ 1] := modl[ 0] * proj[ 1] + modl[ 1] * proj[ 5] + modl[ 2] * proj[ 9] + modl[ 3] * proj[13];
     clip[ 2] := modl[ 0] * proj[ 2] + modl[ 1] * proj[ 6] + modl[ 2] * proj[10] + modl[ 3] * proj[14];
     clip[ 3] := modl[ 0] * proj[ 3] + modl[ 1] * proj[ 7] + modl[ 2] * proj[11] + modl[ 3] * proj[15];

     clip[ 4] := modl[ 4] * proj[ 0] + modl[ 5] * proj[ 4] + modl[ 6] * proj[ 8] + modl[ 7] * proj[12];
     clip[ 5] := modl[ 4] * proj[ 1] + modl[ 5] * proj[ 5] + modl[ 6] * proj[ 9] + modl[ 7] * proj[13];
     clip[ 6] := modl[ 4] * proj[ 2] + modl[ 5] * proj[ 6] + modl[ 6] * proj[10] + modl[ 7] * proj[14];
     clip[ 7] := modl[ 4] * proj[ 3] + modl[ 5] * proj[ 7] + modl[ 6] * proj[11] + modl[ 7] * proj[15];

     clip[ 8] := modl[ 8] * proj[ 0] + modl[ 9] * proj[ 4] + modl[10] * proj[ 8] + modl[11] * proj[12];
     clip[ 9] := modl[ 8] * proj[ 1] + modl[ 9] * proj[ 5] + modl[10] * proj[ 9] + modl[11] * proj[13];
     clip[10] := modl[ 8] * proj[ 2] + modl[ 9] * proj[ 6] + modl[10] * proj[10] + modl[11] * proj[14];
     clip[11] := modl[ 8] * proj[ 3] + modl[ 9] * proj[ 7] + modl[10] * proj[11] + modl[11] * proj[15];

     clip[12] := modl[12] * proj[ 0] + modl[13] * proj[ 4] + modl[14] * proj[ 8] + modl[15] * proj[12];
     clip[13] := modl[12] * proj[ 1] + modl[13] * proj[ 5] + modl[14] * proj[ 9] + modl[15] * proj[13];
     clip[14] := modl[12] * proj[ 2] + modl[13] * proj[ 6] + modl[14] * proj[10] + modl[15] * proj[14];
     clip[15] := modl[12] * proj[ 3] + modl[13] * proj[ 7] + modl[14] * proj[11] + modl[15] * proj[15];


     //Get the right clipping plane
     fFrustum[frustum_RIGHT][plane_A] := clip[ 3] - clip[ 0];
     fFrustum[frustum_RIGHT][plane_B] := clip[ 7] - clip[ 4];
     fFrustum[frustum_RIGHT][plane_C] := clip[11] - clip[ 8];
     fFrustum[frustum_RIGHT][plane_D] := clip[15] - clip[12];
     NormalizePlane(fFrustum, frustum_RIGHT);

     //Get the left clipping plane
     fFrustum[frustum_LEFT][plane_A] := clip[ 3] + clip[ 0];
     fFrustum[frustum_LEFT][plane_B] := clip[ 7] + clip[ 4];
     fFrustum[frustum_LEFT][plane_C] := clip[11] + clip[ 8];
     fFrustum[frustum_LEFT][plane_D] := clip[15] + clip[12];
     NormalizePlane(fFrustum,frustum_LEFT);

     //Get the bottom clipping plane
     fFrustum[frustum_BOTTOM][plane_A] := clip[ 3] + clip[ 1];
     fFrustum[frustum_BOTTOM][plane_B] := clip[ 7] + clip[ 5];
     fFrustum[frustum_BOTTOM][plane_C] := clip[11] + clip[ 9];
     fFrustum[frustum_BOTTOM][plane_D] := clip[15] + clip[13];
     NormalizePlane(fFrustum, frustum_BOTTOM);

     //Get the top clipping plane
     fFrustum[frustum_TOP][plane_A] := clip[ 3] - clip[ 1];
     fFrustum[frustum_TOP][plane_B] := clip[ 7] - clip[ 5];
     fFrustum[frustum_TOP][plane_C] := clip[11] - clip[ 9];
     fFrustum[frustum_TOP][plane_D] := clip[15] - clip[13];
     NormalizePlane(fFrustum, frustum_TOP);

     //Get the back clipping plane
     fFrustum[frustum_BACK][plane_A] := clip[ 3] - clip[ 2];
     fFrustum[frustum_BACK][plane_B] := clip[ 7] - clip[ 6];
     fFrustum[frustum_BACK][plane_C] := clip[11] - clip[10];
     fFrustum[frustum_BACK][plane_D] := clip[15] - clip[14];
     NormalizePlane(fFrustum, frustum_BACK);

     //Get the front clipping plane
     fFrustum[frustum_FRONT][plane_A] := clip[ 3] + clip[ 2];
     fFrustum[frustum_FRONT][plane_B] := clip[ 7] + clip[ 6];
     fFrustum[frustum_FRONT][plane_C] := clip[11] + clip[10];
     fFrustum[frustum_FRONT][plane_D] := clip[15] + clip[14];

     // Normalize the FRONT side
     NormalizePlane(fFrustum, frustum_FRONT);
end;

function TFrustum.CubeInFrustum(aX, aY, aZ, aSize: TFloat): boolean;
var
   i:integer;
   res:array[0..5] of boolean;
begin
     result:=false;
     for i:=0 to 5 do
     begin
          if (fFrustum[i][plane_A] * (aX - aSize) + fFrustum[i][plane_B] * (aY - aSize) + fFrustum[i][plane_C] * (aZ - aSize) + fFrustum[i][plane_D] > 0) then
             continue;
          if (fFrustum[i][plane_A] * (aX + aSize) + fFrustum[i][plane_B] * (aY - aSize) + fFrustum[i][plane_C] * (aZ - aSize) + fFrustum[i][plane_D] > 0) then
             continue;
          if (fFrustum[i][plane_A] * (aX - aSize) + fFrustum[i][plane_B] * (aY + aSize) + fFrustum[i][plane_C] * (aZ - aSize) + fFrustum[i][plane_D] > 0) then
             continue;
          if (fFrustum[i][plane_A] * (aX + aSize) + fFrustum[i][plane_B] * (aY + aSize) + fFrustum[i][plane_C] * (aZ - aSize) + fFrustum[i][plane_D] > 0) then
             continue;
          if (fFrustum[i][plane_A] * (aX - aSize) + fFrustum[i][plane_B] * (aY - aSize) + fFrustum[i][plane_C] * (aZ + aSize) + fFrustum[i][plane_D] > 0) then
             continue;
          if (fFrustum[i][plane_A] * (aX + aSize) + fFrustum[i][plane_B] * (aY - aSize) + fFrustum[i][plane_C] * (aZ + aSize) + fFrustum[i][plane_D] > 0) then
             continue;
          if (fFrustum[i][plane_A] * (aX - aSize) + fFrustum[i][plane_B] * (aY + aSize) + fFrustum[i][plane_C] * (aZ + aSize) + fFrustum[i][plane_D] > 0) then
             continue;
          if (fFrustum[i][plane_A] * (aX + aSize) + fFrustum[i][plane_B] * (aY + aSize) + fFrustum[i][plane_C] * (aZ + aSize) + fFrustum[i][plane_D] > 0) then
             continue;

          exit;
     end;
     result:=true;
end;

function TFrustum.PointInFrustum(aX, aY, aZ: TFloat): boolean;
var
   i:integer;
begin
     result:=false;
     for i:=0 to 5 do
         if (fFrustum[i][plane_A] * aX + fFrustum[i][plane_B] * aY + fFrustum[i][plane_C] * aZ + fFrustum[i][plane_D])<=0 then
            exit;
     result:=true;
end;

function TFrustum.SphereInFrustum(aX, aY, aZ, aRadius: TFloat): boolean;
var
   i:integer;
begin
     result:=false;

     for i:=0 to 5 do
         if (fFrustum[i][plane_A] * aX + fFrustum[i][plane_B] * aY + fFrustum[i][plane_C] * aZ + fFrustum[i][plane_D])<= -aRadius then
            exit;

     result:=true;
end;

end.
