{$WRITEABLECONST ON}
unit GLClass;

interface

uses Forms,
     Classes,
     Windows,
     SysUtils,
     GL,
     GLU,
     BaseTypes;

const
     DoubleBuffer:boolean=true;
     ColorIndex:boolean=false;
     ColorBits:byte=16;
     DepthBits:byte=16;
     StencilBits:byte=8;

type
    TOpenGL=class(TObject)
            private
            fDC:HDC;
            fHRC:HGLRC;
            fPalette:HPALETTE;

            fClearColor:TColorf;

            fFullScreen:boolean;
            fXRes,fYRes:integer;
            fOwner:TForm;
            fFromHandle:boolean;
            fHandle:THandle;
            protected
            procedure MakeCurrent;

            procedure SetClearColor(aColor:TColorf);
            procedure SetFullScreen(v:boolean);

            public
            constructor Create(aOwner:TForm);
            destructor Destroy;override;

            procedure InitGL;
            procedure InitGLFromHandle(aHandle:THandle);
            procedure CleanUpGL;

            procedure Flip;
            procedure ClearAll;
            procedure ClearZBuffer;
            procedure ClearColorBuffer;
            procedure ClearStencilBuffer;

            property DC:HDC read fDC;
            property RC:HGLRC read fHRC;
            property Palette:HPALETTE
                     read fPalette
                     write fPalette;

            property ClearColor:TColorf
                     read fClearColor
                     write SetClearColor;

            property XRes:integer
                     read fXRes
                     write fXRes
                     default 640;
            property YRes:integer
                     read fYRes
                     write fYRes
                     default 480;

            property FullScreen:boolean
                     read fFullScreen
                     write SetFullScreen;
    end;

procedure ChangeRes(rw,rh,rbpp:cardinal);
function GetVendor:string;
function GetRenderer:string;
function GetGLVersion:string;
function GetGLUVersion:string;
function GetExtensions:string;
function GetExtension(num:integer):string;
function GetNumExtensions:integer;

implementation

procedure InitDC(DC: HDC; Palette: HPalette);
var
  hHeap: Integer;
  nColors, i: Integer;
  lpPalette: PLogPalette;
  byRedMask, byGreenMask, byBlueMask: Byte;
  nPixelFormat: Integer;
  pfd: TPixelFormatDescriptor;
begin

  // Initialise the form's DC for OpenGL, and setup a palette.
  FillChar(pfd, SizeOf(pfd), 0);
  with pfd do
  begin
    nSize        := SizeOf(TPixelFormatDescriptor);           // Size of this structure
    nVersion     := 1;                                        // Version number
    dwFlags      := PFD_DRAW_TO_WINDOW or
                    PFD_SUPPORT_OPENGL;                       // Flags
    if DoubleBuffer then
      dwFlags := dwFlags or PFD_DOUBLEBUFFER;                 // Double buffering?
    if ColorIndex then iPixelType := PFD_TYPE_COLORINDEX      // Color index pixel values
    else iPixelType := PFD_TYPE_RGBA;                         // RGBA pixel values
    cColorBits   := ColorBits;
    cDepthBits   := DepthBits;
    cStencilBits := StencilBits;
    iLayerType   := PFD_MAIN_PLANE;                           // Layer type
  end;

  nPixelFormat := ChoosePixelFormat(DC, @pfd);
  SetPixelFormat(DC, nPixelFormat, @pfd);

  DescribePixelFormat(DC, nPixelFormat, SizeOf(TPixelFormatDescriptor), pfd);

  if ((pfd.dwFlags and PFD_NEED_PALETTE) <> 0) then
  begin
    nColors   := 1 shl pfd.cColorBits;
    hHeap     := GetProcessHeap;
    lpPalette := HeapAlloc(hHeap, 0, sizeof(TLogPalette) + (nColors * sizeof(TPaletteEntry)));

    lpPalette^.palVersion := $300;
    lpPalette^.palNumEntries := nColors;

    byRedMask   := (1 shl pfd.cRedBits) - 1;
    byGreenMask := (1 shl pfd.cGreenBits) - 1;
    byBlueMask  := (1 shl pfd.cBlueBits) - 1;

    for i := 0 to nColors - 1 do
    begin
      lpPalette^.palPalEntry[i].peRed   := (((i shr pfd.cRedShift)   and byRedMask)   * 255) DIV byRedMask;
      lpPalette^.palPalEntry[i].peGreen := (((i shr pfd.cGreenShift) and byGreenMask) * 255) DIV byGreenMask;
      lpPalette^.palPalEntry[i].peBlue  := (((i shr pfd.cBlueShift)  and byBlueMask)  * 255) DIV byBlueMask;
      lpPalette^.palPalEntry[i].peFlags := 0;
    end;

    Palette := CreatePalette(lpPalette^);
    HeapFree(hHeap, 0, lpPalette);

    if Palette <> 0 then
    begin
      SelectPalette(DC, Palette, FALSE);
      RealizePalette(DC);
    end;
  end;
  // Restore default flags for the next window or DC.
  DoubleBuffer := TRUE;
  ColorIndex := FALSE;

end;

function ChangeDisplaySettings(lpDevMode: PDeviceModeA; dwFlags: DWORD): Longint; stdcall;
         external user32 name 'ChangeDisplaySettingsA';

procedure ChangeRes(rw, rh, rbpp: Cardinal);
var
  devMode: TDeviceMode;
  modeExists: LongBool;
  modeSwitch, closeMode, i: Integer;
begin

  // Change the display resolution to rw x rh and rbpp bits.
  // Use ChangeRes(0, 0, 0) to restore the normal resolution.
  closeMode := 0;
  i := 0;
  repeat
    modeExists := EnumDisplaySettings(nil, i, devMode);
    // if not modeExists then: This mode may not be supported. We'll try anyway, though.
    with devMode do
    begin
      if (dmBitsPerPel = rbpp) and (dmPelsWidth = rw) and (dmPelsHeight = rh) then
      begin
        modeSwitch := ChangeDisplaySettings(@devMode, CDS_FULLSCREEN);
        if modeSwitch = DISP_CHANGE_SUCCESSFUL then Exit;
      end;
    end;
    if closeMode <> 0 then closeMode := i;
    INC(i);
  until not modeExists;

  EnumDisplaySettings(nil, closeMode, devMode);
  with devMode do
  begin
    dmBitsPerPel := rbpp;
    dmPelsWidth := rw;
    dmPelsHeight := rh;
    dmFields := DM_BITSPERPEL or DM_PELSWIDTH or DM_PELSHEIGHT;
  end;
  modeSwitch := ChangeDisplaySettings(@devMode, CDS_FULLSCREEN);
  if modeSwitch = DISP_CHANGE_SUCCESSFUL then Exit;

  devMode.dmFields := DM_BITSPERPEL;
  modeSwitch := ChangeDisplaySettings(@devMode, CDS_FULLSCREEN);
  if modeSwitch = DISP_CHANGE_SUCCESSFUL then
  begin
    devMode.dmFields := DM_PELSWIDTH or DM_PELSHEIGHT;
    modeSwitch := ChangeDisplaySettings(@devMode, CDS_FULLSCREEN);
    if modeSwitch = DISP_CHANGE_SUCCESSFUL then
    begin
      ChangeDisplaySettings(nil, 0);
      Exit;
    end;
  end;

end;

constructor TOpenGL.Create(aOwner:Tform);
begin
     inherited Create;
     fOwner:=aOwner;
     fXRes:=640;
     fYRes:=480;
end;
destructor TOpenGL.Destroy;
begin
     CleanUpGL;
     inherited Destroy;
end;

procedure TOpenGL.MakeCurrent;
begin
     if (fDC<>0) and (fHRC<>0) then
        wglMakeCurrent(fDC,fHRC);
end;

procedure TOpenGL.InitGL;
begin
     fDC:=GetDC(fOwner.Handle);
     InitDC(fDC,fPalette);
     fHRC:=wglCreateContext(fDC);
     MakeCurrent;
     fFromHandle:=false;
end;
procedure TOpenGL.InitGLFromHandle(aHandle:THandle);
begin
     fDC:=GetDC(aHandle);
     InitDC(fDC,fPalette);
     fHRC:=wglCreateContext(fDC);
     MakeCurrent;
     fFromHandle:=true;
     fHandle:=aHandle;
end;
procedure TOpenGL.CleanUPGL;
begin
     if fFullScreen then
        ChangeRes(0,0,0);
     if fHRC<>0 then
     begin
          wglMakeCurrent(fDC,fHRC);
          wglDeleteContext(fHRC);
     end;
     if fPalette<>0 then DeleteObject(fPalette);
     if not fFromHandle then
        ReleaseDC(fOwner.Handle,fDC)
     else
         ReleaseDC(fHandle,fDC);
end;

procedure TOpenGL.Flip;
begin
     SwapBuffers(fDC);
end;

procedure TOpenGL.SetClearColor(aColor:TColorf);
begin
     fClearColor:=aColor;
     glClearColor(fClearColor.R,
                  fClearColor.G,
                  fClearColor.B,
                  fClearColor.A);
end;

procedure TOpenGL.ClearAll;
begin
     glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT or GL_STENCIL_BUFFER_BIT);
end;
procedure TOpenGL.ClearZBuffer;
begin
     glClear(GL_DEPTH_BUFFER_BIT);
end;
procedure TOpenGL.ClearColorBuffer;
begin
     glClear(GL_COLOR_BUFFER_BIT);
end;
procedure TOpenGL.ClearStencilBuffer;
begin
     glClear(GL_STENCIL_BUFFER_BIT);
end;

procedure TOpenGL.SetFullScreen(v:boolean);
begin
     if v then
     begin
          fFullScreen:=v;
          ChangeRes(fXRes,fYRes,ColorBits);
          fOwner.Left:=0;
          fOwner.Top:=0;
          fOwner.BorderStyle:=bsNone;
          fOwner.WindowState:=wsMaximized;
          fOwner.BringToFront;
     end
     else
         begin
              fFullScreen:=v;
              ChangeREs(0,0,0);
              fOwner.BorderStyle:=bsSingle;
         end;
end;

function GetVendor:string;
begin
     result:=string(glGetString(GL_VENDOR));
end;
function GetRenderer:string;
begin
     result:=string(glGetString(GL_RENDERER));
end;
function GetGLVersion:string;
begin
     result:=string(glGetString(GL_VERSION));
end;
function GetGLUVersion:string;
begin
     result:=string(glGetString(GLU_VERSION));
end;
function GetExtensions:string;
var
   temp:PChar;
begin
     temp:=PChar(glGetString(GL_EXTENSIONS));
     result:=Copy(temp,0,StrLen(temp));
end;
function GetExtension(num:integer):string;
var
   toknum,startpos,endpos,i:integer;
   s:string;
begin
     s:=GetExtensions;
     toknum:=1;
     startpos:=0;
     endpos:=0;
     for i:=1 to Length(s) do
     begin
          if s[i]=' ' then Inc(toknum);
          if toknum=num then
             startpos:=i+1;
          if toknum=num+1 then
             endpos:=i+1;
     end;
     result:=Copy(s,startpos,endpos-startpos);
end;
function GetNumExtensions:integer;
var
   count:integer;
   s:string;
begin
     count:=0;
     result:=0;
     while GetExtension(count)<>'' do
     begin
          s:=GetExtension(count);
          if Trim(s)<>'' then
             Inc(count)
          else
          begin
               result:=count;
               exit;
          end;
     end;
end;
end.
