unit RUVCamera;

interface

uses
    GL,
    BaseTypes,
    BaseCamera,
    BaseMath;

type
    TRUVCamera=class(TCamera)
                             private
                             fViewpoint:TVector;
                             fR:TVector;
                             fU:TVector;
                             fV:TVector;
                             fTargetCamera:boolean;
                             protected
                             public
                             constructor Create;
                             property R:TVector
                                      read fR
                                      write fR;
                             property U:TVector
                                      read fU
                                      write fU;
                             property V:TVector
                                      read fV
                                      write fV;
                             public
                             procedure RotateAboutR(amt:TFloat);
                             procedure RotateAboutU(amt:TFloat);
                             procedure RotateAboutV(amt:TFloat);
                             function GetView:TMatrix;virtual;
                             function GetRotation:TMatrix;override;
                             procedure SetView;override;
                             procedure SetViewOnly;
                             procedure Reset;
                             procedure LookWithMouse(mX,mY:integer);override;
                             property Viewpoint:TVector
                                      read fViewpoint
                                      write fViewpoint;
                             property TargetCamera:boolean
                                      read fTargetCamera
                                      write fTargetCamera;
    end;

implementation

constructor TRUVCamera.Create;
begin
     inherited;
     fR:=MakeVector(1,0,0);
     fU:=MakeVector(0,1,0);
     fV:=MakeVector(0,0,1);
     fTargetCamera:=false;
end;
procedure TRUVCamera.Reset;
begin
     fR:=MakeVector(1,0,0);
     fU:=MakeVector(0,1,0);
     fV:=MakeVector(0,0,1);
     fViewpoint:=MakeVector(0,0,0);
end;
function TRUVCamera.GetView:TMatrix;
var
   rotMatrix,transMatrix:TMatrix;
begin
     transMatrix:=GetIdentity;
     transMatrix[0,3]:=fViewpoint[0];
     transMatrix[1,3]:=fViewpoint[1];
     transMatrix[2,3]:=fViewpoint[2];

     rotMatrix:=GetIdentity;

     rotMatrix[0,0]:=fR[0];
     rotMatrix[1,0]:=fR[1];
     rotMatrix[2,0]:=fR[2];

     rotMatrix[0,1]:=fU[0];
     rotMatrix[1,1]:=fU[1];
     rotMatrix[2,1]:=fU[2];

     rotMatrix[0,2]:=fV[0];
     rotMatrix[1,2]:=fV[1];
     rotMatrix[2,2]:=fV[2];

     if fTargetCamera then
        result:=MatrixMultiply(transMatrix,rotMatrix)
     else
         result:=MatrixMultiply(rotMatrix,transMatrix);
end;
procedure TRUVCamera.SetView;
var
   view:TMatrix;
begin
     inherited;
     view:=GetView;
     view:=MatrixTranspose(view);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity;
     glMultMatrixf(@view[0,0]);     
end;
procedure TRUVCamera.SetViewOnly;
var
   view:TMatrix;
begin
     view:=GetView;
     view:=MatrixTranspose(view);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity;
     glMultMatrixf(@view[0,0]);
end;
procedure TRUVCamera.RotateAboutR(amt:TFloat);
begin
     fR:=VectorRotateX(fR,amt);
     fU:=VectorRotateX(fU,amt);
     fV:=VectorRotateX(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;
procedure TRUVCamera.RotateAboutU(amt:TFloat);
begin
     fR:=VectorRotateY(fR,amt);
     fU:=VectorRotateY(fU,amt);
     fV:=VectorRotateY(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;
procedure TRUVCamera.RotateAboutV(amt:TFloat);
begin
     fR:=VectorRotateZ(fR,amt);
     fU:=VectorRotateZ(fU,amt);
     fV:=VectorRotateZ(fV,amt);
     fR:=VectorNormalize(fR);
     fU:=VectorNormalize(fU);
     fV:=VectorNormalize(fV);
end;

function TRUVCamera.GetRotation: TMatrix;
var
   rotMatrix:TMatrix;
begin
     rotMatrix:=GetIdentity;

     rotMatrix[0,0]:=fR[0];
     rotMatrix[1,0]:=fR[1];
     rotMatrix[2,0]:=fR[2];

     rotMatrix[0,1]:=fU[0];
     rotMatrix[1,1]:=fU[1];
     rotMatrix[2,1]:=fU[2];

     rotMatrix[0,2]:=fV[0];
     rotMatrix[1,2]:=fV[1];
     rotMatrix[2,2]:=fV[2];

     result:=rotMatrix;
end;

procedure TRUVCamera.LookWithMouse(mX, mY: integer);
var
   middleX,middleY:integer;
   rotateX,rotateY:TFloat;
begin
     middleX:=ViewVolume.Width shr 1;
     middleY:=ViewVolume.Height shr 1;

     if (mX=middleX) and (my=middleY) then exit;

     rotateX:=(middleX-mX)/SCALE_MOUSELOOK_X;
     rotateY:=(middleY-mY)/SCALE_MOUSELOOK_Y;

     RotateAboutR(rotateY);
     RotateAboutU(rotateX);
end;

end.
