unit Unit1;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  GL,
  GLUT,
  BaseTypes,
  GLClass,
  GLLight,
  Frustum,
  RUVCamera,
  ExtCtrls;

const
     NUM_SPHERES:integer=1000;

     SPHERE_OBJECT=0;
     CUBE_OBJECT=1;

type
  PTestObject=^TTestObject;
  TTestObject=record
                    Position:TVector;
                    Radius:TFloat;
                    Color:TColorf;
  end;

  TForm1 = class(TForm)
    tmrRender: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure tmrRenderTimer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
    CullingEnabled:boolean;
    oGL:TOpenGL;
    Camera:TRUVCamera;
    Objects:array of TTestObject;
    Fr:TFrustum;
    Light:TLight;
    ObjectType:integer;
    NumDrawn:integer;
    procedure InitObjects;
    procedure SetupGLStates;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
     Randomize;

     oGL:=TOpenGL.Create(Self);
     oGL.FullScreen:=false;
     oGL.InitGL;

     Top:=0;
     Left:=0;

     SetupGLStates;

     Camera:=TRUVCamera.Create;
     Camera.ViewVolume.Width:=ClientWidth;
     Camera.ViewVolume.Height:=ClientHeight;
     Camera.ViewVolume.Yon:=10000;
     Camera.Viewpoint:=MakeVector(0,0,5);
     Camera.TargetCamera:=false;

     Light:=TLight.Create(GL_LIGHT0);
     Light.Enable;
     ObjectType:=SPHERE_OBJECT;

     Fr:=TFrustum.Create;

     SetLength(Objects,NUM_SPHERES);
     InitObjects;
     CullingEnabled:=false;

     ShowCursor(false);
     tmrRender.Enabled:=true;
end;

procedure TForm1.tmrRenderTimer(Sender: TObject);
var
   mPos:TPoint;
   i:integer;
begin
     oGL.ClearAll;

     GetCursorPos(mPos);
     Camera.LookWithMouse(mPos.x,mPos.y);
     Camera.SetView;
     SetCursorPos(ClientWidth shr 1,ClientHeight shr 1);
     Fr.CalculateFrustum;

     if CullingEnabled then
     begin
          NumDrawn:=0;
          for i:=Low(Objects) to High(Objects) do
          begin
               case ObjectType of
                    SPHERE_OBJECT:if Fr.SphereInFrustum(Objects[i].Position[0],Objects[i].Position[1],Objects[i].Position[2],Objects[i].Radius) then
                                  begin
                                       glPushMatrix;
                                       glTranslatef(Objects[i].Position[0],Objects[i].Position[1],Objects[i].Position[2]);
                                       glColor4fv(@Objects[i].Color);
                                       glutSolidSphere(Objects[i].Radius,10,10);
                                       glPopMatrix;
                                       Inc(NumDrawn);
                                  end;
                    CUBE_OBJECT:if Fr.CubeInFrustum(Objects[i].Position[0],Objects[i].Position[1],Objects[i].Position[2],Objects[i].Radius) then
                                  begin
                                       glPushMatrix;
                                       glTranslatef(Objects[i].Position[0],Objects[i].Position[1],Objects[i].Position[2]);
                                       glColor4fv(@Objects[i].Color);
                                       glutSolidCube(Objects[i].Radius);
                                       glPopMatrix;
                                       Inc(NumDrawn);
                                  end;
               end;
          end;
     end
     else
         begin
              NumDrawn:=0;
              for i:=Low(Objects) to High(Objects) do
              begin
                   case ObjectType of
                        SPHERE_OBJECT:begin
                                           glPushMatrix;
                                           glTranslatef(Objects[i].Position[0],Objects[i].Position[1],Objects[i].Position[2]);
                                           glColor4fv(@Objects[i].Color);
                                           glutSolidSphere(Objects[i].Radius,10,10);
                                           glPopMatrix;
                                           Inc(NumDrawn);
                                      end;
                        CUBE_OBJECT:begin
                                           glPushMatrix;
                                           glTranslatef(Objects[i].Position[0],Objects[i].Position[1],Objects[i].Position[2]);
                                           glColor4fv(@Objects[i].Color);
                                           glutSolidCube(Objects[i].Radius);
                                           glPopMatrix;
                                           Inc(NumDrawn);
                                      end;
                   end;
              end;
         end;

     oGL.Flip;
     Caption:=IntToStr(NumDrawn)+' of '+IntToStr(NUM_SPHERES);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
     oGL.Free;
     ShowCursor(true);
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     case Key of
          VK_ESCAPE:begin
                         tmrRender.Enabled:=false;
                         Close;
          end;
     end;
end;

procedure TForm1.InitObjects;
var
   i:integer;
begin
     for i:=Low(Objects) to High(Objects) do
     begin
          Objects[i].Position:=MakeVector(Random*100,Random*100,-Random*100);
          Objects[i].Radius:=Random;
          Objects[i].Color:=MakeColorf(Random,Random,Random,1);
     end;
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
     case UpCase(Key) of
          'C':CullingEnabled:=not CullingEnabled;
          '0':ObjectType:=0;
          '1':ObjectType:=1;
     end;
end;

procedure TForm1.SetupGLStates;
begin
     glEnable(GL_LIGHTING);
     glEnable(GL_DEPTH_TEST);
     glEnable(GL_COLOR_MATERIAL);
end;

end.
